// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file MultiGridPreconWrapper.h */

#ifndef AMDIS_MULTIGRIDPRECONWRAPPER_H
#define AMDIS_MULTIGRIDPRECONWRAPPER_H

#include "Preconditioner.h"
#include "MemoryManager.h"

namespace AMDiS {

  template<typename T> class DOFVector;
  template<typename M, typename V> class MultiGridSolverBase;

  // =========================================================================
  // ===== MGPreconWrqpperScal ===============================================
  // =========================================================================

  /** \brief
   * \ingroup Solver
   * Wrapper for using multigrid as preconditioner within other iterative 
   * solvers.
   */
  class MGPreconWrapperScal : public PreconditionerScal
  {
  public:
    /** \brief
     * Constructor.
     */
    MGPreconWrapperScal(::std::string name, int size_ = 1, int row_ = 0);

    /** \brief
     * Destructor.
     */
    virtual ~MGPreconWrapperScal();

    /** \brief
     * realisation of Preconditioner::init
     */
    void init();

    /** \brief
     * realisation of Preconditioner::precon
     */
    void precon(DOFVector<double> *vec);

    /** \brief
     * realisation of Preconditioner::exit
     */
    void exit();

    // =======================================================================
    // ===== Creator =========================================================
    // =======================================================================

    class Creator : public PreconditionerScalCreator
    { 
    public:
      MEMORY_MANAGED(Creator);

      /** \brief
       * Creates a DiagonalPreconditioner.
       */
      PreconditionerScal *create() { 
	return NEW MGPreconWrapperScal(name_, size, row);
      };      
    };

  protected:
    MultiGridSolverBase<DOFMatrix, DOFVector<double> > *mgSolver_;
    DOFVector<double> *solution_;
    bool initMG_;
  };

}

#endif
